@@ -26,6 +26,12 @@
 #ifndef QS_UTIL_H
 #define QS_UTIL_H
 
+/* ----------------------------------
+ * version info
+ * ---------------------------------- */
+static const char man_version[] = "9.74";
+static const char man_date[] = "04 November 2011";
+
 /* ----------------------------------
  * definitions
  * ---------------------------------- */
@@ -33,7 +39,6 @@
 #define CR 13
 #define LF 10
 
-
 /* ----------------------------------
  * structures
  * ---------------------------------- */
@@ -48,6 +53,10 @@ typedef struct qs_event_st {
 /* ----------------------------------
  * functions
  * ---------------------------------- */
+char *qs_CMD(char *cmd);
+void qs_man_print(int man, const char *fmt, ...);
+void qs_man_println(int man, const char *fmt, ...);
+
 /* io */
 int qs_getLine(char *s, int n);
 int qs_getLinef(char *s, int n, FILE *f);
