@@ -25,7 +25,7 @@
  *
  */
 
-static const char revision[] = "$Id: qslog.c,v 1.12 2011/03/29 17:56:50 pbuchbinder Exp $";
+static const char revision[] = "$Id: qslog.c,v 1.24 2011/11/01 22:11:13 pbuchbinder Exp $";
 
 #include <stdio.h>
 #include <string.h>
@@ -41,8 +41,6 @@ static const char revision[] = "$Id: qslog.c,v 1.12 2011/03/29 17:56:50 pbuchbin
 #include <regex.h>
 #include <time.h>
 
-//#include <config.h>
-
 #include "qs_util.h"
 
 /* ----------------------------------
@@ -92,8 +90,10 @@ static char  m_file_name[MAX_LINE];
 static int   m_rotate = 0;
 /* regex to search the time string */
 static regex_t m_trx;
+static regex_t m_trx2;
 /* real time mode (default) or offline */
 static int   m_offline = 0;
+static int   m_offline_data = 0;
 static char  m_date_str[MAX_LINE];
 static int   m_mem = 0;
 /* debug */
@@ -121,32 +121,73 @@ static char *skipElement(const char* line) {
   char delim = p[0];
   if(delim == '\'' || delim == '\"') {
     p++;
-    while(p[0] != delim && p[0] != 0) {
+    while(p[0] != delim && p[0] != 0 && p[-1] != '\\') {
       p++;
     }
+    p++;
+  } else {
+    char *eq = NULL;
+    if(m_offline) {
+      // offline mode: check for <name>='<value>' entry
+      eq = strstr(p, "='");
+      if(eq && (eq - p) < 10) {
+	// near hit
+	p = &eq[3];
+	while(p[0] != '\'' && p[0] != 0 && p[-1] != '\\') {
+	  p++;
+	}
+	p++;
+      } else {
+	// something else...
+	eq=NULL;
+      }
+    }
+    if(!eq) {
+      while(p[0] != ' ' && p[0] != 0) {
+	p++;
+      }
+    }
   }
-  while(p[0] != ' ' && p[0] != 0) {
+  while(p[0] == ' ') {
     p++;
   }
-  if(p[0] == ' ') p++;
   return p;
 }
 
+static void stripNum(char **p) {
+  char *s = *p;
+  int len;
+  while(s && (s[0] < '0' || s[0] > '9')) {
+    s++;
+  }
+  len = strlen(s);
+  while(len > 0 && (s[len] < '0' || s[len] > '9')) {
+    s[len] = '\0';
+    len--;
+  }
+  *p = s;
+}
+
 /*
  * get and cut an element
  */
 static char *cutNext(char **line) {
   char *c = *line;
   char *p = skipElement(*line);
+  char delim;
   *line = p;
   if(p[0]) {
     p--; p[0] = '\0';
   }
   /* cut leading and tailing " */
-  if(c[0] == '\"') {
+  delim = c[0];
+  if(delim == '\'' || delim == '\"') {
+    int len;
     c++;
-    if(c[strlen(c)-1] == '\"') {
+    len = strlen(c);
+    while(len > 0 && c[strlen(c)-1] == delim) {
       c[strlen(c)-1] = '\0';
+      len--;
     }
   }
   return c;
@@ -339,6 +380,7 @@ static void printAndResetStat(char *timeStr) {
 	    av[0], mem[0] ? mem : "-");
   } else {
     fprintf(m_f, "sl;-;m;-");
+    m_offline_data = 0;
   }
   fprintf(m_f, "\n");
   qs_csUnLock();
@@ -359,6 +401,8 @@ static void printAndResetStat(char *timeStr) {
  */
 static void updateStat(const char *cstr, char *line) {
   char *T = NULL; /* time */
+  char *t = NULL; /* time ms */
+  char *D = NULL; /* time us */
   char *S = NULL; /* status */
   char *BI = NULL; /* bytes in */
   char *B = NULL; /* bytes */
@@ -368,6 +412,7 @@ static void updateStat(const char *cstr, char *line) {
   char *Q = NULL; /* mod_qos event message */
   const char *c = cstr;
   char *l = line;
+  long tme;
   if(!line[0]) return;
   while(c[0]) {
     /* process known types */
@@ -379,6 +424,14 @@ static void updateStat(const char *cstr, char *line) {
       if(l != NULL && l[0] != '\0') {
         T = cutNext(&l);
       }
+    } else if(c[0] == 't') {
+      if(l != NULL && l[0] != '\0') {
+        t = cutNext(&l);
+      }
+    } else if(c[0] == 'D') {
+      if(l != NULL && l[0] != '\0') {
+        D = cutNext(&l);
+      }
     } else if(c[0] == 'S') {
       if(l != NULL && l[0] != '\0') {
         S = cutNext(&l);
@@ -417,20 +470,8 @@ static void updateStat(const char *cstr, char *line) {
     }
     c++;
   }
-  if(m_offline && m_verbose) {
-    m_lines++;
-    printf("[%ld] I=%s U=%s B=%s i=%s S=%s T=%s Q=%s\n", m_lines,
-	   I == NULL ? "(null)" : I,
-	   U == NULL ? "(null)" : U,
-	   B == NULL ? "(null)" : B,
-	   BI == NULL ? "(null)" : BI,
-	   S == NULL ? "(null)" : S,
-	   T == NULL ? "(null)" : T,
-	   Q == NULL ? "(null)" : Q
-	   );
-  }
-  qs_csLock();
 
+  qs_csLock();
   if(Q != NULL) {
     if(strchr(Q, 'V') != NULL) {
       m_qos_v++;
@@ -464,13 +505,16 @@ static void updateStat(const char *cstr, char *line) {
   }
   if(B != NULL) {
     /* transferred bytes */
+    stripNum(&B);
     m_byte_count = m_byte_count + atoi(B);
   }
   if(BI != NULL) {
     /* transferred bytes */
+    stripNum(&BI);
     m_i_byte_count = m_i_byte_count + atoi(BI);
   }
   if(S != NULL) {
+    stripNum(&S);
     if(S[0] == '1') {
       m_status_1++;
     } else if(S[0] == '1') {
@@ -481,13 +525,22 @@ static void updateStat(const char *cstr, char *line) {
       m_status_3++;
     } else if(S[0] == '4') {
       m_status_4++;
-    } else {
+    } else if(S[0] == '5') {
       m_status_5++;
     }
   }
-  if(T != NULL) {
+  if(T != NULL || t != NULL || D != NULL) {
     /* response duration */
-    int tme = atoi(T);
+    if(T) {
+      stripNum(&T);
+      tme = atol(T);
+    } else if(t) {
+      stripNum(&t);
+      tme = atol(t) / 1000;
+    } else if(D) {
+      stripNum(&D);
+      tme = atol(D) / 1000000;
+    }
     m_duration_count = m_duration_count + tme;
     if(tme < 1) {
       m_duration_0++;
@@ -508,6 +561,19 @@ static void updateStat(const char *cstr, char *line) {
   /* request counter */
   m_line_count++;
   qs_csUnLock();
+
+  if(m_offline && m_verbose) {
+    m_lines++;
+    printf("[%ld] I=%s U=%s B=%s i=%s S=%s T=%ld Q=%s\n", m_lines,
+	   I == NULL ? "(null)" : I,
+	   U == NULL ? "(null)" : U,
+	   B == NULL ? "(null)" : B,
+	   BI == NULL ? "(null)" : BI,
+	   S == NULL ? "(null)" : S,
+	   tme,
+	   Q == NULL ? "(null)" : Q
+	   );
+  }
   line[0] = '\0';
 }
 
@@ -536,7 +602,43 @@ static int mstr2i(const char *m) {
 static time_t getMinutes(char *line) {
   regmatch_t ma;
   if(regexec(&m_trx, line, 1, &ma, 0) != 0) {
-    return 0;
+    if(regexec(&m_trx2, line, 1, &ma, 0) == 0) {
+      time_t minutes = 0;
+      int buf_len = ma.rm_eo - ma.rm_so + 1;
+      char buf[buf_len];
+      strncpy(buf, &line[ma.rm_so], ma.rm_eo - ma.rm_so);
+      buf[ma.rm_eo - ma.rm_so] = '\0';
+      /* yyyy mm dd hh:mm:ss,mmm */
+      /* cut seconds */
+      buf[strlen(buf)-7] = '\0';
+      /* get minutes */
+      minutes = minutes + (atoi(&buf[strlen(buf)-2]));
+      /* cut minutes */
+      buf[strlen(buf)-3] = '\0';
+      /* get hours */
+      minutes = minutes + (atoi(&buf[strlen(buf)-2]) * 60);
+      /* store date information */
+      {
+	char *year;
+	char *month;
+	char *day;
+	/* cut hours */
+	buf[strlen(buf)-3] = '\0';
+	day = &buf[strlen(buf)-2];
+	/* cut day */
+	buf[strlen(buf)-3] = '\0';
+	month = &buf[strlen(buf)-2];
+	/* cut month */
+	buf[strlen(buf)-3] = '\0';
+	year = buf;
+	snprintf(m_date_str, sizeof(m_date_str), "%s.%s.%s", day, month, year);
+      }
+      return minutes;
+    } else {
+      // unknown format
+      fprintf(stdout, "E");
+      return 0;
+    }
   } else {
     time_t minutes = 0;
     int buf_len = ma.rm_eo - ma.rm_so + 1;
@@ -597,6 +699,7 @@ static void readStdinOffline(const char *cstr) {
   time_t unitTime = 0;
   while(qs_getLine(line, sizeof(line))) {
     time_t l_time = getMinutes(line);
+    m_offline_data = 1;
     if(unitTime == 0) {
       unitTime = l_time;
       qs_setTime(unitTime * 60);
@@ -606,10 +709,15 @@ static void readStdinOffline(const char *cstr) {
     } if(l_time < unitTime) {
       /* leap in time... */
       updateStat(cstr, line);
+      fprintf(stdout, "X");
+      fflush(stdout);
+      unitTime = 0;
     } else {
-      if(!m_verbose) {
-	fprintf(stdout, ".");
-	fflush(stdout);
+      if(l_time > unitTime) {
+      	if(!m_verbose) {
+	  fprintf(stdout, ".");
+	  fflush(stdout);
+	}
       }
       while(l_time > unitTime) {
 	snprintf(buf, sizeof(buf), "%s %.2ld:%.2ld:00", m_date_str, unitTime/60, unitTime%60);
@@ -620,6 +728,10 @@ static void readStdinOffline(const char *cstr) {
       updateStat(cstr, line);
     }
   }
+  if(m_offline_data) {
+    snprintf(buf, sizeof(buf), "%s %.2ld:%.2ld:00", m_date_str, unitTime/60, unitTime%60);
+    printAndResetStat(buf);
+  }
 }
 
 /*
@@ -658,74 +770,138 @@ static void *loggerThread(void *argv) {
   return NULL;
 }
 
-static void usage(char *cmd) {
+static void usage(char *cmd, int man) {
+  if(man) {
+    //.TH [name of program] [section number] [center footer] [left footer] [center header]
+    printf(".TH %s 1 \"%s\" \"mod_qos utilities %s\" \"%s man page\"\n", qs_CMD(cmd), man_date,
+	   man_version, cmd);
+  }
   printf("\n");
-  printf("Utility to collect request statistics from access log data.\n");
+  if(man) {
+    printf(".SH NAME\n");
+  }
+  qs_man_print(man, "%s - collects request statistics from access log data.\n", cmd);
   printf("\n");
-  printf("Usage: %s -f <format_string> -o <out_file> [-p [-v]] [-x] [-u <name>] [-m]\n", cmd);
+  if(man) {
+    printf(".SH SYNOPSIS\n");
+  }
+  qs_man_print(man, "%s%s -f <format_string> -o <out_file> [-p [-v]] [-x] [-u <name>] [-m]\n", man ? "" : "Usage: ", cmd);
   printf("\n");
-  printf("Summary\n");
-  printf("%s is a real time access log analyzer. It collects\n", cmd);
-  printf("the data from stdin. The output is written to the specified\n");
-  printf("file every minute. The output includes the following entries:\n");
-  printf("  - requests per second ("NRS")\n");
-  printf("  - bytes sent to the client per second ("NBS")\n");
-  printf("  - bytes received from the client per second ("NBIS")\n");
-  printf("  - repsonse status codes within the last minute (1xx,2xx,3xx,4xx,5xx)\n");
-  printf("  - average response duration ("NAV")\n");
-  printf("  - distribution of response durations within the last minute\n");
-  printf("    (<1s,1s,2s,3s,4s,5s,>5)\n");
-  printf("  - average system load (sl)\n");
-  printf("  - free memory (m) (not available for all platforms)\n");
-  printf("  - number of client ip addresses seen withn the last %d seconds (ip)\n", ACTIVE_TIME);
-  printf("  - number of different users seen withn the last %d seconds (usr)\n", ACTIVE_TIME);
-  printf("  - number of mod_qos events within the last minute (qV=create session,\n");
-  printf("    qS=session pass, qD=access denied, qK=connection closed, qT=dynamic\n");
-  printf("    keep-alive, qL=request/response slow down, qs=serialized request)\n");
+  if(man) {
+    printf(".SH DESCRIPTION\n");
+  } else {
+    printf("Summary\n");
+  }
+  qs_man_print(man, "%s is a real time access log analyzer. It collects\n", cmd);
+  qs_man_print(man, "the data from stdin. The output is written to the specified\n");
+  qs_man_println(man, "file every minute and includes the following entries:\n");
+  qs_man_println(man, "  - requests per second ("NRS")\n");
+  qs_man_println(man, "  - bytes sent to the client per second ("NBS")\n");
+  qs_man_println(man, "  - bytes received from the client per second ("NBIS")\n");
+  qs_man_println(man, "  - repsonse status codes within the last minute (1xx,2xx,3xx,4xx,5xx)\n");
+  qs_man_println(man, "  - average response duration ("NAV")\n");
+  qs_man_println(man, "  - distribution of response durations within the last minute\n");
+  qs_man_print(man, "    (<1s,1s,2s,3s,4s,5s,>5)\n");
+  if(man) printf("\n");
+  qs_man_println(man, "  - average system load (sl)\n");
+  qs_man_println(man, "  - free memory (m) (not available for all platforms)\n");
+  qs_man_println(man, "  - number of client ip addresses seen withn the last %d seconds (ip)\n", ACTIVE_TIME);
+  qs_man_println(man, "  - number of different users seen withn the last %d seconds (usr)\n", ACTIVE_TIME);
+  qs_man_println(man, "  - number of mod_qos events within the last minute (qV=create session,\n");
+  qs_man_print(man, "    qS=session pass, qD=access denied, qK=connection closed, qT=dynamic\n");
+  qs_man_print(man, "    keep-alive, qL=request/response slow down, qs=serialized request)\n");
   printf("\n");
-  printf("Options\n");
-  printf("  -f <format_string>\n");
-  printf("     Defines the log data format and the positions of data\n");
-  printf("     elements processed by this utility.\n");
-  printf("     See to the 'LogFormat' directive of the httpd.conf file\n");
-  printf("     to see the format defintions of the servers access log\n");
-  printf("     data. %s knows the following elements:\n", cmd);
-  printf("     I defines the client ip address (%%h)\n");
-  printf("     R defines the request line (%%r)\n");
-  printf("     S defines HTTP response status code (%%s)\n");
-  printf("     B defines the transferred bytes (%%b or %%O)\n");
-  printf("     i defines the received bytes (%%I)\n");
-  printf("     T defines the request duration (%%T)\n");
-  printf("     U defines the user tracking id (%%{mod_qos_user_id}e)\n");
-  printf("     Q defines the mod_qos_ev event message (%%{mod_qos_ev}o)\n");
-  printf("     . defines an element to ignore (unknown string)\n");
-  printf("  -o <out_file>\n");
-  printf("     Specifies the file to store the output to.\n");
-  printf("  -p\n");
-  printf("     Used when reading the log data from a file (cat/pipe). %s is\n", cmd);
-  printf("     started using it's offline mode in order to process existing log\n");
-  printf("     files (post processing).\n");
-  printf("  -v\n");
-  printf("     Verbose mode.\n");
-  printf("  -x\n");
-  printf("     Rotates the output file once a day (move).\n");
-  printf("  -u <name>\n");
-  printf("     Become another user, e.g. www-data.\n");
-  printf("  -m\n");
-  printf("     Calculates free system memory every minute.\n");
+  if(man) {
+    printf(".SH OPTIONS\n");
+  } else {
+    printf("Options\n");
+  }
+  if(man) printf(".TP\n");
+  qs_man_print(man, "  -f <format_string>\n");
+  if(man) printf("\n");
+  qs_man_print(man, "     Defines the log data format and the positions of data\n");
+  qs_man_print(man, "     elements processed by this utility.\n");
+  qs_man_print(man, "     See to the 'LogFormat' directive of the httpd.conf file\n");
+  qs_man_print(man, "     to see the format defintions of the servers access log data.\n");
+  if(man) printf("\n");
+  qs_man_println(man, "     %s knows the following elements:\n", cmd);
+  qs_man_println(man, "     I defines the client ip address (%%h)\n");
+  qs_man_println(man, "     R defines the request line (%%r)\n");
+  qs_man_println(man, "     S defines HTTP response status code (%%s)\n");
+  qs_man_println(man, "     B defines the transferred bytes (%%b or %%O)\n");
+  qs_man_println(man, "     i defines the received bytes (%%I)\n");
+  qs_man_println(man, "     T defines the request duration (%%T)\n");
+  qs_man_println(man, "     t defines the request duration in milliseconds (optionally used instead of T)\n");
+  qs_man_println(man, "     D defines the request duration in microseconds (optionally used instead of T)\n");
+  qs_man_println(man, "     U defines the user tracking id (%%{mod_qos_user_id}e)\n");
+  qs_man_println(man, "     Q defines the mod_qos_ev event message (%%{mod_qos_ev}e)\n");
+  qs_man_println(man, "     . defines an element to ignore (unknown string)\n");
+  if(man) printf("\n.TP\n");
+  qs_man_print(man, "  -o <out_file>\n");
+  if(man) printf("\n");
+  qs_man_print(man, "     Specifies the file to store the output to.\n");
+  if(man) printf("\n.TP\n");
+  qs_man_print(man, "  -p\n");
+  if(man) printf("\n");
+  qs_man_print(man, "     Used when reading the log data from a file (cat/pipe). %s is\n", cmd);
+  qs_man_print(man, "     started using it's offline mode in order to process existing log\n");
+  qs_man_print(man, "     files (post processing).\n");
+  if(man) printf("\n.TP\n");
+  qs_man_print(man, "  -v\n");
+  if(man) printf("\n");
+  qs_man_print(man, "     Verbose mode.\n");
+  if(man) printf("\n.TP\n");
+  qs_man_print(man, "  -x\n");
+  if(man) printf("\n");
+  qs_man_print(man, "     Rotates the output file once a day (move).\n");
+  if(man) printf("\n.TP\n");
+  qs_man_print(man, "  -u <name>\n");
+  if(man) printf("\n");
+  qs_man_print(man, "     Become another user, e.g. www-data.\n");
+  if(man) printf("\n.TP\n");
+  qs_man_print(man, "  -m\n");
+  if(man) printf("\n");
+  qs_man_print(man, "     Calculates free system memory every minute.\n");
   printf("\n");
-  printf("Example configuration using pipped logging:\n");
-  printf("  LogFormat \"%%t %%h \\\"%%r\\\" %%>s %%b \\\"%%{User-Agent}i\\\" %%T\"\n");
-  printf("  TransferLog \"|./bin/%s -f ..IRSB.T -o ./logs/stat_log\"\n", cmd);
+  if(man) {
+    printf(".SH EXAMPLE\n");
+    printf("Configuration using pipped logging:\n");
+    printf("\n");
+  } else {
+    printf("Example configuration using pipped logging:\n");
+  }
+  qs_man_println(man, "  LogFormat \"%%t %%h \\\"%%r\\\" %%>s %%b \\\"%%{User-Agent}i\\\" %%T\"\n");
+  qs_man_println(man, "  TransferLog \"|./bin/%s -f ..IRSB.T -x -o ./logs/stat_log\"\n", cmd);
   printf("\n");
-  printf("Example configuration using the CustomLog directive:\n");
-  printf("  CustomLog \"|./bin/%s -f ISBT -o ./logs/stat_log\" \"%%h %%>s %%b %%T\"\n", cmd);
+  if(man) {
+    printf("Configuration using the CustomLog directive:\n");
+    printf("\n");
+  } else {
+   printf("Example configuration using the CustomLog directive:\n");
+  }
+  qs_man_println(man, "  CustomLog \"|./bin/%s -f ISBTQ -x -o ./logs/stat_log\" \"%%h %%>s %%b %%T %%{mod_qos_ev}e\"\n", cmd);
   printf("\n");
-  printf("Example for post processing:\n");
-  printf("  cat access_log | ./bin/%s -f ..IRSB.T -o ./logs/stat_log -p\n", cmd);
+  if(man) {
+    printf("Post processing:\n");
+    printf("\n");
+  } else {
+    printf("Example for post processing:\n");
+  }
+  qs_man_println(man, "  cat access_log | ./bin/%s -f ..IRSB.T -o ./logs/stat_log -p\n", cmd);
   printf("\n");
-  printf("See http://opensource.adnovum.ch/mod_qos/ for further details.\n");
-  exit(1);
+  if(man) {
+    printf(".SH SEE ALSO\n");
+    printf("qsexec(1), qsfilter2(1), qsgrep(1), qspng(1), qsrotate(1), qssign(1), qstail(1)\n");
+    printf(".SH AUTHOR\n");
+    printf("Pascal Buchbinder, http://opensource.adnovum.ch/mod_qos/\n");
+  } else {
+    printf("See http://opensource.adnovum.ch/mod_qos/ for further details.\n");
+  }
+  if(man) {
+    exit(0);
+  } else {
+    exit(1);
+  }
 }
 
 int main(int argc, char **argv) {
@@ -771,11 +947,13 @@ int main(int argc, char **argv) {
     } else if(strcmp(*argv,"-x") == 0) { /* activate log rotation */
       m_rotate = 1;
     } else if(strcmp(*argv,"-h") == 0) {
-      usage(cmd);
+      usage(cmd, 0);
     } else if(strcmp(*argv,"--help") == 0) {
-      usage(cmd);
+      usage(cmd, 0);
     } else if(strcmp(*argv,"-?") == 0) {
-      usage(cmd);
+      usage(cmd, 0);
+    } else if(strcmp(*argv,"--man") == 0) {
+      usage(cmd, 1);
     } else {
       qerror("unknown option '%s'", *argv);
       exit(1);
@@ -784,7 +962,7 @@ int main(int argc, char **argv) {
     argv++;
   }
   /* requires at leas an output file and a format string */
-  if(file == NULL || config == NULL) usage(cmd);
+  if(file == NULL || config == NULL) usage(cmd, 0);
 
   if(username && getuid() == 0) {
     struct passwd *pwd = getpwnam(username);
@@ -819,12 +997,20 @@ int main(int argc, char **argv) {
 
   if(m_offline) {
     nice(10);
-    /* init time pattern regex */
-    regcomp(&m_trx, "[0-9]{2}/[a-zA-Z]{3}/[0-9]{4}:[0-9]{2}:[0-9]{2}:[0-9]{2}", REG_EXTENDED);
+    /* init time pattern regex, std apache access log */
+    regcomp(&m_trx, 
+	    "[0-9]{2}/[a-zA-Z]{3}/[0-9]{4}:[0-9]{2}:[0-9]{2}:[0-9]{2}",
+	    REG_EXTENDED);
+    /* other time patterns: yyyy mm dd hh:mm:ss,mmm */
+    regcomp(&m_trx2, 
+	    "[0-9]{4}[ -]{1}[0-9]{2}[ -]{1}[0-9]{2} [0-9]{2}:[0-9]{2}:[0-9]{2}[,.]{1}[0-9]{3}",
+	    REG_EXTENDED);
     fprintf(stderr, "[%s]: offline mode (writes to %s)\n", cmd, file);
     m_date_str[0] = '\0';
     readStdinOffline(config);
-    fprintf(stdout, "\n");
+    if(!m_verbose) {
+      fprintf(stdout, "\n");
+    }
   } else {
     pthread_create(&tid, tha, loggerThread, NULL);
     readStdin(config);
@@ -832,4 +1018,3 @@ int main(int argc, char **argv) {
   fclose(m_f);
   return 0;
 }
-
