@@ -23,7 +23,7 @@
  *
  */
 
-static const char revision[] = "$Id: qspng.c,v 1.7 2011/03/04 16:56:14 pbuchbinder Exp $";
+static const char revision[] = "$Id: qspng.c,v 1.10 2011/11/01 22:11:13 pbuchbinder Exp $";
 
 #include <stdio.h>
 #include <strings.h>
@@ -67,9 +67,14 @@ static const qs_png_elt_t qs_png_elts[] = {
   { "3s", "requests with 3 seconds response time", 25, 90, 180 },
   { "4s", "requests with 4 seconds response time", 25, 95, 180 },
   { "5s", "requests with 5 seconds response time", 15, 90, 180 },
-  { ">5s", "requests slower than 5 seconds", 35, 90, 185 },
+  { ">5s","requests slower than 5 seconds", 35, 90, 185 },
+  { "1xx","requets with HTTP status 1xx",   50, 70, 150 },
+  { "2xx","requets with HTTP status 2xx",   50, 70, 150 },
+  { "3xx","requets with HTTP status 3xx",   50, 70, 150 },
+  { "4xx","requets with HTTP status 4xx",   50, 70, 150 },
+  { "5xx","requets with HTTP status 5xx",   50, 70, 150 },
   { "ip", "IP addresses",          55, 60, 150 },
-  { "usr", "active users",         55, 66, 150 },
+  { "usr","active users",          55, 66, 150 },
   { "qv", "created VIP sessions",  55, 50, 155 },
   { "qs", "session pass",          55, 75, 160 },
   { "qd", "access denied",         55, 70, 170 },
@@ -542,29 +547,67 @@ static long qs_png_draw(int width, int height, int border,
 }
 
 
-static void usage(char *cmd) {
+static void usage(char *cmd, int man) {
+  if(man) {
+    //.TH [name of program] [section number] [center footer] [left footer] [center header]
+    printf(".TH %s 1 \"%s\" \"mod_qos utilities %s\" \"%s man page\"\n", qs_CMD(cmd), man_date,
+	   man_version, cmd);
+  }
   printf("\n");
-  printf("Utility to draw a png graph from qslog output data.\n");
+  if(man) {
+    printf(".SH NAME\n");
+    qs_man_print(man, "Utility to draw a png graph from qslog(1) output data.\n");
+  } else {
+    qs_man_print(man, "Utility to draw a png graph from qslog output data.\n");
+  }
   printf("\n");
-  printf("Usage: %s -i <stat_log_file> -p <parameter> -o <out_file> [-10]\n", cmd);
+  if(man) {
+    printf(".SH SYNOPSIS\n");
+  }
+  qs_man_print(man, "%s%s -i <stat_log_file> -p <parameter> -o <out_file> [-10]\n", man ? "" : "Usage: ", cmd);
   printf("\n");
-  printf("Summary\n");
-  printf("%s is a tool to generate png (portable network graphics)\n", cmd);
-  printf("raster images files from semicolon separated data generated by the\n");
-  printf("qslog utility. It reads up to the first 1440 entries (24 hours)\n");
-  printf("and prints a graph using the values defined by the 'parameter' \n");
-  printf("name.\n");
+  if(man) {
+    printf(".SH DESCRIPTION\n");
+  } else {
+    printf("Summary\n");
+  }
+  qs_man_print(man, "%s is a tool to generate png (portable network graphics)\n", cmd);
+  qs_man_print(man, "raster images files from semicolon separated data generated by the\n");
+  qs_man_print(man, "qslog utility. It reads up to the first 1440 entries (24 hours)\n");
+  qs_man_print(man, "and prints a graph using the values defined by the 'parameter' \n");
+  qs_man_print(man, "name.\n");
   printf("\n");
-  printf("Options\n");
-  printf("  -i <stats_log_file>\n");
-  printf("     Input file to read data from.\n");
-  printf("  -p <parameter>\n");
-  printf("     Parameter name, e.g. r/s or usr.\n");
-  printf("  -o <out_file>\n");
-  printf("     Output file name, e.g. stat.png.\n");
-  printf("\n");
-  printf("See http://opensource.adnovum.ch/mod_qos/ for further details.\n");
-  exit(1);
+  if(man) {
+    printf(".SH OPTIONS\n");
+  } else {
+    printf("Options\n");
+  }
+  if(man) printf(".TP\n");
+  qs_man_print(man, "  -i <stats_log_file>\n");
+  if(man) printf("\n");
+  qs_man_print(man, "     Input file to read data from.\n");
+  if(man) printf("\n.TP\n");
+  qs_man_print(man, "  -p <parameter>\n");
+  if(man) printf("\n");
+  qs_man_print(man, "     Parameter name, e.g. r/s or usr.\n");
+  if(man) printf("\n.TP\n");
+  qs_man_print(man, "  -o <out_file>\n");
+  if(man) printf("\n");
+  qs_man_print(man, "     Output file name, e.g. stat.png.\n");
+  if(man) {
+    printf(".SH SEE ALSO\n");
+    printf("qsexec(1), qsfilter2(1), qsgrep(1), qslog(1), qsrotate(1), qssign(1), qstail(1)\n");
+    printf(".SH AUTHOR\n");
+    printf("Pascal Buchbinder, http://opensource.adnovum.ch/mod_qos/\n");
+  } else {
+    printf("\n");
+    printf("See http://opensource.adnovum.ch/mod_qos/ for further details.\n");
+  }
+  if(man) {
+    exit(0);
+  } else {
+    exit(1);
+  }
 }
 
 int main(int argc, char **argv) {
@@ -613,13 +656,21 @@ int main(int argc, char **argv) {
       if (--argc >= 1) {
 	out = *(++argv);
       }
+    } else if(strcmp(*argv,"-h") == 0) {
+      usage(cmd, 0);
+    } else if(strcmp(*argv,"--help") == 0) {
+      usage(cmd, 0);
+    } else if(strcmp(*argv,"-?") == 0) {
+      usage(cmd, 0);
+    } else if(strcmp(*argv,"--man") == 0) {
+      usage(cmd, 1);
     }
     argc--;
     argv++;
   }
   
   
-  if(infile == NULL || param == NULL || out == NULL) usage(cmd);
+  if(infile == NULL || param == NULL || out == NULL) usage(cmd, 0);
   for(elt = qs_png_elts; elt->param != NULL ; ++elt) {
     if(strcmp(elt->param, param) == 0) {
       name = elt->name;
